#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gnome.h>
#include "vroom.h"
#include "defines.h"
#include "levels.h"
#include "callbacks.h"

extern struct _level *cur_level;
extern struct _vroom* _curroom;
extern int cntr;
extern struct _vroom* head;


void create_new_room(int pos)
{
	struct _level *old_level;
	struct _vroom *myvroom;
	char buf[300];
	if (pos>0) old_level=cur_level->prev;
	if (pos<0) old_level=cur_level->next;

	myvroom=new_vroom();
	sprintf(buf,"%i",cntr);
	new_room(myvroom,buf,_curroom->x,_curroom->y);

	head->prev=myvroom;
	myvroom->prev=0;
	myvroom->next=head;
	head=myvroom;

	

	if (pos>0)
	{
		show_down(myvroom);
		show_up(_curroom);
		myvroom->down=_curroom;
		_curroom->up=myvroom;
	}

	if (pos<0)
	{
		show_down(_curroom);
		show_up(myvroom);
		myvroom->up=_curroom;
		_curroom->down=myvroom;
	}

	set_curroom(myvroom);
	cntr++;
	
}


struct _vroom* new_vroom()
{
	struct _vroom* tmproom;
	tmproom=(struct _vroom*) malloc((size_t)sizeof(struct _vroom));
	return tmproom;
}

int new_room(struct _vroom* cur_room, char *id, double x, double y)
{
	int i;
	strcpy(cur_room->id,id);
	
	cur_room->rlevel=cur_level;


	cur_room->rectangle=gnome_canvas_item_new(
		cur_level->group, GNOME_TYPE_CANVAS_RECT,
		"x1",(double)x,
		"y1",(double)y,
		"x2",(double)(x+RSIZE2),
		"y2",(double)(y+RSIZE2),
		"outline_color",RECT_COLOR,
		NULL);

	cur_room->text=gnome_canvas_item_new(
		cur_level->group, GNOME_TYPE_CANVAS_TEXT,
		"x",(double)x+RSIZE1,
		"y",(double)y+RSIZE1,
		"text",id,
		"justification",GTK_JUSTIFY_CENTER,
		"fill_color",TEXT_COLOR,
		NULL);
	
	for (i=0; i<8; i++)
	{
		cur_room->points[i]=gnome_canvas_points_new(2);
		cur_room->points[i]->coords[0]=0.0;
		cur_room->points[i]->coords[1]=0.0;
		cur_room->points[i]->coords[2]=0.0;
		cur_room->points[i]->coords[3]=0.0;
		cur_room->rexit[i]=0;
		cur_room->room_ptr[i]=0;
	}

	cur_room->x=x;
	cur_room->y=y;
	cur_room->next=0;
	cur_room->prev=0;
	cur_room->up=0;
	cur_room->down=0;
	cur_room->vup=0;
	cur_room->vdown=0;
	cur_room->head_exit=0;
	cur_room->edited=0;
	for (i=0; i<8; i++)
	{
		cur_room->room_ptr[i]=0;
	}
}

int show_up(struct _vroom *troom)
{

	 if (!troom->vup) troom->vup=gnome_canvas_item_new(
	 	troom->rlevel->group, GNOME_TYPE_CANVAS_TEXT,
		"x",(double)troom->x+RSIZE1/2,
		"y",(double)troom->y+RSIZE1/2,
		"text","+",
		"fill-color",EXIT_COLOR,NULL);
}

int show_down(struct _vroom *troom)
{

	if (!troom->vdown) troom->vdown=gnome_canvas_item_new(
	 	troom->rlevel->group, GNOME_TYPE_CANVAS_TEXT,
		"x",(double)troom->x+3*RSIZE1/2,
		"y",(double)troom->y+3*RSIZE1/2,
		"text","-",
		"fill-color",EXIT_COLOR,NULL);
}


int set_exits(struct _vroom *cur_room, double daexits[8])
{	
	int add_x1[]={0,RSIZE1,RSIZE2,0,RSIZE2,0,RSIZE1,RSIZE2};
	 int add_y1[]={RSIZE2 ,RSIZE2,RSIZE2,RSIZE1 ,RSIZE1,0,  0,0};
	int add_x2[]={-RSIZE2,RSIZE1,RSIZE3,-RSIZE2,RSIZE3,-RSIZE2,RSIZE1,RSIZE3};
	int add_y2[]={RSIZE3, RSIZE3,RSIZE3,RSIZE1,RSIZE1,-RSIZE2,-RSIZE2,-RSIZE2};
	
	int i;
        int j;
        int cx,cy;
        cx=cur_room->x; cy=cur_room->y;
        for (i=0; i<8; i++)
        {
                if (daexits[i]!=0 && daexits[i]!=-1 && !cur_room->rexit[i])
                {
                        cur_room->points[i]->coords[0]=cx+add_x1[i];
                        cur_room->points[i]->coords[1]=cy+add_y1[i];
                        cur_room->points[i]->coords[2]=cx+add_x2[i];
                        cur_room->points[i]->coords[3]=cy+add_y2[i];
                        cur_room->rexit[i]=gnome_canvas_item_new(
                        cur_level->group,
                        GNOME_TYPE_CANVAS_LINE,
                        "points",cur_room->points[i],
			"fill_color",EXIT_COLOR,
			"last_arrowhead",DO_ARROWS,
			"arrow_shape_a",3.0,
			"arrow_shape_b",6.0,
			"arrow_shape_c",3.0,
			"smooth",TRUE,
			NULL);
                }
	}
}


int link_rooms(struct _vroom* r1, struct _vroom* r2,double dax[])
{
	int i;
	double dx;

	for (i=0; i<8; i++)
	{
		if (dax[i] && dax[i]!=-1) {
		r2->room_ptr[i]=r1;
		r1->room_ptr[7-i]=r2;
		} 	
	}

	
}


struct _vroom* get_room_by_id(int rid)
{
	struct _vroom *tmpvr;

	tmpvr=head;

	while (tmpvr!=0)
	{
		if (atoi(tmpvr->id)==rid) return tmpvr;
		tmpvr=tmpvr->next;
	}
//	g_print("\n-----------------------------------------------\n");
//	g_print("You have found a bug!\nPlease report it by sending mail to frostedcmos@yahoo.com\n");
//	g_print("-----------------------------------------------\n");

//	mdialog_msg("Fatal Error: You have found a bug\nPlease report it to frostedcmos@yahoo.com\nThanks.");
	
	return 0;
}
